<?php
/*------------------------------------------------------------------------
# Copyright (C) 2012-2015 WebxSolution Ltd. All Rights Reserved.
# @license - GPLv2.0
# Author: WebxSolution Ltd
# Websites:  http://www.webxsolution.com
# Terms of Use: An extension that is derived from the JoomlaCK editor will only be allowed under the following conditions: http://joomlackeditor.com/terms-of-use
# ------------------------------------------------------------------------*/ 

defined('_JEXEC') or die;

/**
 *Ark inline content Article Context Class
 *
 * @package     Inlinecontent.Contexts
 * @subpackage  Inlinecontent.contexts.FlexicontentArticle
 */
 
 require_once (JPATH_SITE.'/components/com_flexicontent/classes/flexicontent.helper.php');
 
 
class ARKContextsFlexicontentArticle extends ARKContextsBase
{


	public function __construct($id)
	{
		JTable::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_flexicontent/tables'); 
        $this->table = 	JTable::getInstance('flexicontent_items','');
		$this->table->load($id);
		parent::__construct($id);		
	}	


    public function get()
	{
	
		if($this->id == null)
			return parent::get();	
     	
		$this->table->articletext = $this->table->introtext;

		if (!empty($this->table->fulltext))
		{
			$this->table->articletext .= '<hr id="system-readmore" />'.chr(13).chr(13).$this->table->fulltext;
		}
		return parent::get($pk);	
	}
	
	
	
	public function triggerContentPlugins($rawText)
	{
		
		$item = new stdclass;
					
		$text = '';
		
		if (isset($rawText))
		{
			$pattern = '#<hr\s+id=("|\')system-readmore("|\')\s*\/*>#i';
			$tagPos = preg_match($pattern, $rawText);
			
			if ($tagPos == 0)
			{
				
				$text = $rawText;
			}
			else
			{
				list ($text, $rawText) = preg_split($pattern, $rawText, 2);
				$text = $text.$rawText;
			}
		}

		$item->text = $text;

		$params = $this->getParams();
		$params->set('inline',false);
		$dispatcher	= JEventDispatcher::getInstance();
			
		$suppress_arr = array('jcomments', 'jom_comment_bot');
		FLEXIUtilities::suppressPlugins($suppress_arr, 'suppress' );
		
		JPluginHelper::importPlugin('content');
		$dispatcher->trigger('onContentPrepare', array ('com_content.article', &$item, &$params, 0));
	
		// Restore suppressed plugins
		FLEXIUtilities::suppressPlugins($suppress_arr, 'restore' );
		
		return array( 'data'=>$item->text);
	}
			
	public function save($data,$type = 'body')
	{
		if($this->id == null)
			return array( 'title'=>'','data'=>'');	
		if($type == 'title')
		{
			$data['title'] = strip_tags($data['title']); 
			$data['title'] = html_entity_decode($data['title']);
		}
		if(isset($data['articletext']))
		{	
			$data['articletext'] = base64_decode($data['articletext']);	
		
			$pattern = '#<hr\s+id=("|\')system-readmore("|\')\s*\/*>#i';
			$tagPos	= preg_match($pattern, $data['articletext']);
			
			if ($tagPos == 0)	{
				$data['introtext']= $data['articletext'];
				$data['fulltext']  = '';
			} else 	{
				list($data['introtext'], $data['fulltext']) = preg_split($pattern, $data['articletext'], 2);
				$data['fulltext']  = JString::strlen( trim($data['fulltext'] ) ) ? $data['fulltext']  : '';
			}
		}  
		$app = JFactory::getApplication();
		$app->input->set("isflexicontent", "yes");
		
		// We need to fake joomla's states ... when triggering the before?after save content event
		$fc_state = $this->table->state;
		if ( in_array($fc_state, array(1,-5)) ) $jm_state = 1;           // published states
		else if ( in_array($fc_state, array(0,-3,-4)) ) $jm_state = 0;   // unpublished states
		else $jm_state = $fc_state;                                      // trashed & arch
	
		$datenow = JFactory::getDate();
		$this->table->modified = $datenow->toSQL();
		$this->table->modified_by = JFactory::getUser()->get('id');
		
		$last_version = FLEXIUtilities::getLastVersions($this->id, true);
		$current_version = FLEXIUtilities::getCurrentVersions($this->id, true);
		
		$dispatcher	= JEventDispatcher::getInstance();
		
		$this->table->version = $last_version+1;
		
		JPluginHelper::importPlugin('flexicontent');
		JPluginHelper::importPlugin('content');
		
		
		// Some compatibility steps

		$view = $app->input->get('view', false);
		$app->input->set('view', 'article');	  
		$app->input->set('option', 'com_content');
		
		$this->table->state = $jm_state;
		$dispatcher->trigger('onBeforeContentSave', array('com_content.article', &$this->table, false));
			
		$this->table->save($data);
		
		$dispatcher->trigger('onAfterContentSave', array('com_content.article', &$this->table, false));
		
		$app->input->set('view', 'article');	  
		$app->input->set('option', 'com_content');
		
		// Reverse compatibility steps
		$db = JFactory::getDBO();
        $query = $db->getQuery(true);
		$query->update($db->quoteName('#__content'))->set('state = '. $fc_state )->where('id = '.$this->id );
 		$db->setQuery($query);
		$db->query();
		
		$app->input->set('view', $view);
		$app->input->set('option', 'com_flexicontent');
		
		//We will always be using versioning for inline editing
		
		//Create and store version METADATA information
		
		$version = new stdClass();
        $version->item_id		= (int)$this->table->id;
		$version->version_id	= (int)$last_version+1;
		$version->created		= $this->table->modified;
		$version->created_by 	= $this->table->modified_by;
		$version->comment		= '';
		$db->insertObject('#__flexicontent_versions', $version);
		
		
		//Now duplicate lastversion for fields for consistency

		$sql = 'INSERT INTO #__flexicontent_items_versions(version,field_id,item_id,valueorder,value)';
		
		$query->clear()
			->select(array($last_version+1,'field_id','item_id','valueorder','value'))
			->from('#__flexicontent_items_versions')
			->where('version'.' = '. (int)$last_version)
			->where('item_id'.' = '. (int)$this->id);
			
		$db->setQuery($sql.' '.$query);
		$db->query();	
		
		

		if($type == 'title')
		{	
			$query->clear()
				->select('id')
				->from('#__flexicontent_fields')
				->where($db->quoteName('name').' = '.$db->quote('title'))
				->where($db->quoteName('iscore').'= 1');
			$db->setQuery($query);
			$field_id = $db->loadResult();	
			
			$query->clear()
				->update($db->quoteName('#__flexicontent_items_versions'))->set($db->quoteName('value').' = '. $db->quote($data['title']))->where('field_id = '. (int) $field_id);
				$db->setQuery($query);
				$db->query();
		}
		else
		{	
			$query->clear()
				->select('id')
				->from('#__flexicontent_fields')
				->where($db->quoteName('name').' = '.$db->quote('text'))
				->where($db->quoteName('iscore').'= 1');
			$db->setQuery($query);
			$field_id = $db->loadResult();	
			
			$query->clear()
				->update($db->quoteName('#__flexicontent_items_versions'))->set($db->quoteName('value').' = '. $db->quote($data['articletext']))->where('field_id = '. (int) $field_id)
				->where('version'.' = '. (int)$last_version+1)
				->where('item_id'.' = '. (int)$this->id);
				$db->setQuery($query);
				$db->query();
		}
		
		$dispatcher->trigger('onCompleteSaveItem', array( &$this->table, &$fields ));
		
		//Let's save version manually as we cannot use JTableObserverContenthistory in this case!
		$typeAlias = 'com_flexicontent.flexicontent';	
		$contenthistoryHelper = new JHelperContenthistory($typeAlias);			
		$contenthistoryHelper->store($this->table);
		
		//We need to process data as we are sending it back to the client
		
		$item = $this->table;
		
		$params = $this->getParams();
		$params->set('inline',false);
		
		if(!$item->fulltext)
		{	
			$item->text = $item->introtext;
		}	
		elseif ($params->get('show_intro', '1') == '1')
		{
			$item->text = $item->introtext.chr(13).chr(13).$item->fulltext;
		}
		elseif ($item->fulltext)
		{
			$item->text = $item->fulltext;
		}
		else
		{
			$item->text = $item->introtext;
		}
		
		//let's detect if any plugin tags are beig used 
		//if so let's inform the system to warn the user
		$message = $this->detectPluginTags($item->text);
				
		$suppress_arr = array('jcomments', 'jom_comment_bot');
		FLEXIUtilities::suppressPlugins($suppress_arr, 'suppress' );
		
		$dispatcher->trigger('onContentPrepare', array ('com_content.article', &$item, &$params, 0));
	
		// Restore suppressed plugins
		FLEXIUtilities::suppressPlugins($suppress_arr, 'restore' );
												
		return array( 'title'=>$item->title,'data'=>$item->text,'message'=>$message);	
	}
	
	public function version($versionId,$type)
	{
				
		$historyTable = JTable::getInstance('Contenthistory');
		$historyTable->load($versionId);
		$rowArray = JArrayHelper::fromObject(json_decode($historyTable->version_data));
			
		$item = $this->table;

		$item->bind($rowArray);	
		if($type == 'title')
		{
			return array( 'data'=>$item->title);
		}
		$text = '';
		

		
		$text = $item->introtext;
		if (!empty($item->fulltext))
		{
			$text .= '<hr id="system-readmore" />'.chr(13).chr(13).$item->fulltext;
		}
	
	
		return array( 'data'=>$text);
		
	}
	
	protected function getParams()
	{
		
		$app = JFactory::getApplication();
		$menu = $app->getMenu()->getActive();  // Retrieve currently active menu item (NOTE: this applies when Itemid variable or menu item alias exists in the URL)
		jimport('joomla.html.parameter');
		
		
		// **********************************************************************
		// Retrieve RELATED parameters that will be merged into item's parameters
		// **********************************************************************
		
			
		// Retrieve parameters of current category (NOTE: this applies when cid variable exists in the URL)
		$catParams = '';
		$cid = 0;
		$db = JFactory::getDBO();
		$query = $db->getQuery(true);
		
		if(!$app->input->get('task',false))
			$cid = $app->input->get('cid',0);
		
		if ( $cid ) {
			$query->select('params')
				->from('#__categories')
				->where('id = ' . (int) $cid);
			$db->setQuery($query);
			$catParams = $db->loadResult();	
		}
		
		// Retrieve item's Content Type parameters
		$query->clear()
			->select('t.attribs')
			->from('#__flexicontent_types AS t')
			->leftjoin('#__flexicontent_items_ext AS ie ON ie.type_id = t.id')
			->where('ie.item_id = ' . (int)$this->_id);
		$db->setQuery($query);
		$typeParams = $db->loadResult();	
		
		// ***************************************************************************************************
		// Merge parameters in order: component, menu, (item 's) current category, (item's) content type, item
		// ***************************************************************************************************
		
		// Get the COMPONENT only parameters, then merge the menu parameters
		$comp_params = JComponentHelper::getComponent('com_flexicontent')->params;
		$params = clone $comp_params; 
		
		if ($menu) {
			$menu_params = $menu->params; 
			$params->merge($menu_params);
		}
		
		// b. Merge parameters from current category
		$catParams = new JRegistry($catParams); 
		$catParams->set('show_title', '');       // Prevent show_title from propagating ... to the item, it is meant for category view only
		$catParams->set('title_linkable', '');   // Prevent title_linkable from propagating ... to the item, it is meant for category view only
		$catParams->set('show_editbutton', '');  // Prevent title_linkable from propagating ... to the item, it is meant for category view only
		$params->merge($catParams);
		
		// c. Merge TYPE parameters into the page configuration
		$typeParams = new JRegistry($typeParams);
		$params->merge($typeParams);

		// d. Merge ITEM parameters into the page configuration
		$itemparams = new JRegistry($this->table->attribs);
		$params->merge($itemparams);

		// e. Merge ACCESS permissions into the page configuration
		
			
		// Compute EDIT access permissions.
		$accessperms =  new JRegistry;
		$asset	= 'com_content.article.'.$this->table->id;
		$user = JFactory::getUser();
		if ($user->authorise('core.edit', $asset)) {
			$accessperms->set('access-edit', true);
		}elseif ( $user->authorise('core.edit.own', $asset) && $user->get('id') == $this->table->created_by)
		{
			$accessperms->set('access-edit', true);
		}
		
		// Compute EDIT STATE access permissions.
		
		if ($user->authorise('core.edit.state', $asset)) {
			$accessperms->set('access-edit-state', true);
		}elseif ( $user->authorise('core.edit.state.own', $asset) && $user->get('id') == $this->table->created_by)
		{
			$accessperms->set('access-edit-state', true);
		}
		
		// Compute DELETE access permissions.
		
		// first check delete permission on the item
		if ($user->authorise('core.delete', $asset)) {
			$accessperms->set('access-delete', true);
		}elseif( $user->authorise('core.delete.own', $asset) && $user->get('id') == $this->table->created_by)
		{
			$accessperms->set('access-delete', true);
		}
		
		$params->merge($accessperms);
		
		return $params;
	}
	
}